﻿using BankContracts.BindingModels;
using BankContracts.BusinessLogicsContracts;
using BankContracts.SearchModels;
using BankContracts.StoragesContracts;
using BankContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BankBusinessLogic.BusinessLogics
{
    public class ClientLogic : IClientLogic
    {
        private readonly int _loginMaxLength = 50;
        private readonly int _passwordMaxLength = 50;
        private readonly int _passwordMinLength = 10;

        private readonly ILogger _logger;
        private readonly IClientStorage _clientStorage;

        public ClientLogic(ILogger<ClientLogic> logger, IClientStorage headwaiterStorage)
        {
            _logger = logger;
            _clientStorage = headwaiterStorage;
        }

        public bool Create(ClientBindingModel model)
        {
            CheckModel(model);

            if (_clientStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");

                return false;
            }

            return true;
        }

        public bool Delete(ClientBindingModel model)
        {
            CheckModel(model, false);

            _logger.LogInformation("Delete. Id: {Id}", model.Id);

            if (_clientStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");

                return false;
            }

            return true;
        }

        public ClientViewModel? ReadElement(ClientSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            _logger.LogInformation("ReadElement. HeadwaiterFIO: {HeadwaiterFIO}. HeadwaiterLogin: {HeadwaiterLogin}. Id: {Id}.", model.ClientFIO, model.ClientLogin, model.Id);

            var element = _clientStorage.GetElement(model);

            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }

            _logger.LogInformation("ReadElement find. Id: {Id}", element.Id);

            return element;
        }

        public List<ClientViewModel>? ReadList(ClientSearchModel? model)
        {
            _logger.LogInformation("ReadList. HeadwaiterFIO: {HeadwaiterFIO}. HeadwaiterLogin: {HeadwaiterLogin}. Id: {Id}.", model?.ClientFIO, model?.ClientLogin, model?.Id);

            var list = model == null ? _clientStorage.GetFullList() : _clientStorage.GetFilteredList(model);

            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }

            _logger.LogInformation("ReadList. Count: {Count}", list.Count);

            return list;
        }

        public bool Update(ClientBindingModel model)
        {
            CheckModel(model);

            if (_clientStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(ClientBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (!withParams)
            {
                return;
            }

            if (string.IsNullOrEmpty(model.ClientFIO))
            {
                throw new ArgumentNullException("Нет ФИО метродотеля", nameof(model.ClientFIO));
            }

            if (string.IsNullOrEmpty(model.ClientLogin))
            {
                throw new ArgumentNullException("Нет логина метродотеля", nameof(model.ClientLogin));
            }

            if (model.ClientLogin.Length > _loginMaxLength)
            {
                throw new ArgumentNullException("Логин слишком длинный", nameof(model.ClientLogin));
            }

            if (string.IsNullOrEmpty(model.ClientNumber))
            {
                throw new ArgumentNullException("Нет номера телефона метродотеля", nameof(model.ClientNumber));
            }

            if (string.IsNullOrEmpty(model.ClientEmail))
            {
                throw new ArgumentNullException("Нет почты метродотеля", nameof(model.ClientEmail));
            }

            if (string.IsNullOrEmpty(model.ClientPassword))
            {
                throw new ArgumentNullException("Нет пароля метродотеля", nameof(model.ClientPassword));
            }

            if (model.ClientPassword.Length < _passwordMinLength)
            {
                throw new ArgumentNullException("Пароль слишком короткий", nameof(model.ClientPassword));
            }

            if (model.ClientPassword.Length > _passwordMaxLength)
            {
                throw new ArgumentNullException("Пароль слишком длинный", nameof(model.ClientPassword));
            }

            _logger.LogInformation("Headwaiter. HeadwaiterFIO: {HeadwaiterFIO}. HeadwaiterLogin: {HeadwaiterLogin}. Id: {Id}", model.ClientFIO, model.ClientLogin, model.Id);

            var element = _clientStorage.GetElement(new ClientSearchModel
            {
                ClientLogin = model.ClientLogin
            });

            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("метродотель с таким логином уже есть");
            }
        }
    }
}
